@@ -1,10 +1,12 @@
+
 package Net::OpenSSH::Parallel;
 
-our $VERSION = '0.11';
+our $VERSION = '0.14';
 
 use strict;
 use warnings;
 use Carp qw(croak carp);
+our @CARP_NOT=qw(Net::OpenSSH);
 
 use Net::OpenSSH;
 use Net::OpenSSH::Parallel::Constants qw(:error :on_error);
@@ -19,7 +21,7 @@ sub new {
     my ($class, %opts) = @_;
     my $max_workers = delete $opts{workers};
     my $max_conns = delete $opts{connections};
-    my $max_reconns = delete $opts{reconnections};
+    my $reconnections = delete $opts{reconnections};
     my $on_error = delete $opts{on_error};
 
     if ($max_conns) {
@@ -55,7 +57,7 @@ sub new {
 		 max_workers => $max_workers,
 		 max_conns => $max_conns,
 		 num_conns => 0,
-		 max_reconns => $max_reconns,
+		 reconnections => $reconnections,
 		 on_error => $on_error,
 	       };
     bless $self, $class;
@@ -82,9 +84,10 @@ sub add_host {
     $label =~ /([,*!()<>\/{}])/ and croak "invalid char '$1' in host label";
     my %opts = (@_ & 1 ? (host => @_) : @_);
     $opts{host} = $label unless defined $opts{host};
+    $opts{batch_mode} = 1 unless defined $opts{batch_mode};
 
     my $on_error = delete $opts{on_error};
-    my $max_reconns = delete $opts{reconnections};
+    my $reconnections = delete $opts{reconnections};
 
     my $host = { label => $label,
 		 workers => 1,
@@ -93,7 +96,7 @@ sub add_host {
 		 state => 'done',
 		 queue => [],
 		 on_error => $on_error,
-		 max_reconns => $max_reconns,
+		 reconnections => $reconnections,
 	       };
 
     $self->{hosts}{$label} = $host;
@@ -182,7 +185,7 @@ sub push {
     my $alias = $push_action_alias{$action};
     $action = $alias if defined $alias;
 
-    $action =~ /^(?:command|(?:(?:rsync|scp)_(?:get|put))|join|sub|parsub|here|stop|goto|_notify)$/
+    $action =~ /^(?:command|(?:(?:rsync|scp)_(?:get|put))|join|sub|parsub|here|stop|goto|_notify|connect)$/
 	or croak "bad action '$action'";
 
     my %opts = (($action ne 'sub' and ref $_[0] eq 'HASH') ? %{shift()} : ());
@@ -263,6 +266,7 @@ sub _at_error {
     my ($self, $label, $error) = @_;
     my $host = $self->{hosts}{$label};
     my $task = delete $host->{current_task};
+    my $queue = $host->{queue};
 
     $debug and _debug(error => "_at_error label: $label, error: $error");
 
@@ -271,10 +275,15 @@ sub _at_error {
 
     my $on_error;
     if ($error == OSSH_MASTER_FAILED) {
-	my $max_reconns = _hash_chain_get(max_reconns => $host, $self) || 0;
-	my $reconns = $host->{current_task_reconns}++ || 0;
-	$debug and _debug(error => "[$label] reconnection: $reconns, max: $max_reconns");
-	if ($reconns < $max_reconns) {
+        if ($host->{state} eq 'connecting') {
+            # task is not set in state connecting!
+            $task and die "internal error: task is defined in state connecting";
+            $opts = $queue->[0][1] if @$queue;
+        }
+	my $max_reconnections = _hash_chain_get(reconnections => $opts, $host, $self) || 0;
+	my $reconnections = $host->{current_task_reconnections}++ || 0;
+	$debug and _debug(error => "[$label] reconnection: $reconnections, max: $max_reconnections");
+	if ($reconnections < $max_reconnections) {
 	    $debug and _debug(error => "[$label] will reconnect!");
 	    $on_error = OSSH_ON_ERROR_RETRY;
 	}
@@ -295,8 +304,6 @@ sub _at_error {
 
     $debug and _debug(error => "[$label] on_error (final): $on_error, error: $error (".($error+0).")");
 
-    my $queue = $host->{queue};
-
     if ($on_error == OSSH_ON_ERROR_RETRY) {
 	if ($error == OSSH_MASTER_FAILED) {
 	    $self->_set_host_state($label, 'suspended');
@@ -315,21 +322,21 @@ sub _at_error {
 	return;
     }
 
-    delete $host->{current_task_reconns};
+    delete $host->{current_task_reconnections};
 
     if ($on_error == OSSH_ON_ERROR_IGNORE) {
-	if ($error == OSSH_JOIN_FAILED) {
-	    $self->_set_host_state($label, 'ready');
-	}
-	elsif ($error == OSSH_MASTER_FAILED) {
-	    # stablishing a new connection failed, what we should do?
-	    # currently we remove the next task from the queue and
+	if ($error == OSSH_MASTER_FAILED) {
+	    # establishing a new connection failed, what we should do?
+	    # currently we remove the current task from the queue and
 	    # continue.
-	    shift @$queue;
+	    shift @$queue unless $task;
 	    $self->_set_host_state($label, 'suspended');
 	    $self->_disconnect_host($label);
 	    $self->_set_host_state($label, 'ready');
 	}
+        else {
+	    $self->_set_host_state($label, 'ready');
+        }
 	# else do nothing
     }
     else {
@@ -454,14 +461,14 @@ sub _disconnect_any_host {
     $self->_disconnect_host($label);
 }
 
-my @private_opts = qw(on_error or_goto);
+my @private_opts = qw(on_error or_goto reconnections);
 
 sub _at_ready {
     my ($self, $label) = @_;
     if (my $max_workers = $self->{max_workers}) {
 	my $in_state = $self->{in_state};
 	my $num_workers = $self->_num_workers;
-	$debug and _debug(workers => "num workers: $num_workers, maximun: $max_workers");
+	$debug and _debug(workers => "num workers: $num_workers, maximum: $max_workers");
 	if ($num_workers > $max_workers) {
 	    $debug and _debug(workers => "[$label] suspending");
 	    $self->_set_host_state($label, 'suspended');
@@ -544,6 +551,8 @@ sub _at_ready {
                 }
             }
 
+            next if $action eq 'connect';
+
             $host->{current_task} = [$action, @$task];
             my %opts = %{shift @$task};
             delete @opts{@private_opts};
@@ -690,7 +699,7 @@ sub _finish_task {
         $self->_set_host_state($label, 'ready');
         $self->_skip($label, $or_goto) if defined $or_goto;
         delete $host->{current_task};
-        delete $host->{current_task_reconns};
+        delete $host->{current_task_reconnections};
     }
     else {
 	my $label = delete $self->{ssh_master_by_pid}{$pid};
@@ -890,7 +899,7 @@ everything!
 
 =back
 
-=head2 Labelling hosts
+=head2 Labeling hosts
 
 Every host is identified by an unique label that is given when the
 host is registered into the parallel scheduler. Usually, the host
@@ -901,7 +910,7 @@ The rationale behind using labels is that a hostname does not
 necessarily identify unique "remote processors" (for instance,
 sometimes your logical "remote processors" may be user accounts
 distributed over a set of hosts: C<foo1@bar1>, C<foo2@bar1>,
-C<foo3@bar2>, ...; a set of hosts that are accesible behind an unique
+C<foo3@bar2>, ...; a set of hosts that are accessible behind an unique
 IP, listening in different ports; etc.)
 
 =head2 Selecting hosts
@@ -925,7 +934,7 @@ Other possible selectors are:
   '*doz*'               # everything containing 'doz'
 
 I<Note: I am still considering how the selector mini-language should
-be, don't hesitate to send your suggestions!>
+be, do not hesitate to send your suggestions!>
 
 =head2 Local resource usage
 
@@ -939,9 +948,9 @@ command) that results in around 5MB of RAM usage per host.
 CPU usage varies greatly depending on the tasks carried out. The most
 expensive are short remote tasks (because of the local process
 creation and destruction overhead) and tasks that transfer big
-ammounts of data through SSH (because of the encryption going on).
+amounts of data through SSH (because of the encryption going on).
 
-In practice, CPU usage doesn't matter too much (mostly because the OS
+In practice, CPU usage does not matter too much (mostly because the OS
 would be able to manage it but also because there is not too many
 things we can do to reduce it) and usually it is RAM about what we
 should be more concerned.
@@ -952,7 +961,7 @@ The module accepts two parameters to limit resource usage:
 
 =item * C<workers>
 
-is the maximun number of remote commands that can be running
+is the maximum number of remote commands that can be running
 concurrently.
 
 =item * C<connections>
@@ -967,8 +976,8 @@ limits RAM usage and limiting the the maximum number of workers
 indirectly limits CPU usage.
 
 The module requires the maximum number of connections to be at least
-equal or bigger than the maximun number of workers, and it is
-recomended that C<maximum_connections E<gt>= 2 * maximum_workers>
+equal or bigger than the maximum number of workers, and it is
+recommended that C<maximum_connections E<gt>= 2 * maximum_workers>
 (otherwise the scheduler will not be able to reuse connections
 efficiently).
 
@@ -1063,10 +1072,10 @@ constants and the corresponding policy will be applied.
 
 =head3 Retrying connection errors
 
-If the module fails when trying to stablish a new SSH connection or
+If the module fails when trying to establish a new SSH connection or
 when an existing connection dies unexpectedly, the option
 C<reconnections> can be used to instruct the module to retry the
-connection until it succeds or the given maximun is reached.
+connection until it succeeds or the given maximum is reached.
 
 C<reconnections> is accepted by both the L</new> and L</add_host>
 methods.
@@ -1099,7 +1108,7 @@ parallel (see L</Local resource usage>).
 
 =item connections => $maximum_connections
 
-sets the maximum number of SSH connections that can be stablished
+sets the maximum number of SSH connections that can be established
 simultaneously (see L</Local resource usage>).
 
 $maximum_connections must be equal or bigger than $maximum_workers
@@ -1107,8 +1116,8 @@ $maximum_connections must be equal or bigger than $maximum_workers
 =item reconnections => $maximum_reconnections
 
 when connecting to some host fails, this argument tells the module the
-maximum number of additional connection atemps that it should perform
-before giving up. The default value is zero.
+maximum number of additional connection attempts that it should
+perform before giving up. The default value is zero.
 
 See also L</Retrying connection errors>.
 
@@ -1122,11 +1131,11 @@ Sets the error handling policy (see L</Error handling>).
 
 =item $pssh->add_host($label, $host, %opts)
 
-registers a new host into the C<$pssh> object.
+X<add_host>registers a new host into the C<$pssh> object.
 
 C<$label> is the name used to refer to the registered host afterwards.
 
-When the hostname argument is ommited, the label is used also as the
+When the hostname argument is omitted, the label is used also as the
 hostname.
 
 The accepted options are:
@@ -1137,14 +1146,17 @@ The accepted options are:
 
 Sets the error handling policy (see L</Error handling>).
 
-=item max_reconns => $maximum_reconnections
+=item reconnections => $maximum_reconnections
 
-See </Retrying connection errors>.
+See L</Retrying connection errors>.
 
 =back
 
 Any additional option will be passed verbatim to the L<Net::OpenSSH>
-constructor later.
+constructor later. For instance:
+
+  $pssh->add_host($host, user => $user, password => $password);
+
 
 =item $pssh->push($selector, $action, \%opts, @action_args)
 
@@ -1170,7 +1182,7 @@ Example:
 
 =item scp_put => @local, $remote
 
-These methods queue an SCP remote file copy operation in the selected
+These methods queue a C<scp> remote file copy operation in the selected
 hosts.
 
 =item rsync_get => @remote, $local
@@ -1265,7 +1277,7 @@ One common usage is to synchronize all servers at some point:
   $ssh->push('*', join => '*');
 
 By default, errors are propagated at joins. For instance, in the
-example above, if the scp_get operation queued on server_B failed, it
+example above, if the C<scp_get> operation queued on server_B failed, it
 would abort any further operation queued on server_B and any further
 operation queued after the join in server_A. See also L</Error
 handling>.
@@ -1280,7 +1292,7 @@ operations.
 Jumps forward until the given C<here> tag is reached.
 
 Joins to other hosts queues will be ignored, and joins from other
-queues to this one will be succesfully fulfilled. For instance:
+queues to this one will be successfully fulfilled. For instance:
 
   $pssh->add_host(A => ...);
   $pssh->add_host(B => ...);
@@ -1292,7 +1304,7 @@ queues to this one will be succesfully fulfilled. For instance:
   $pssh->push('*', here => 'there');
   $pssh->push('*', cmd  => 'echo "bye bye from %HOST%");
 
-Note that it is not possible to jump backguards.
+Note that it is not possible to jump backwards.
 
 There is an special target C<END> that can be used to jump to the end
 of the queue.
@@ -1306,6 +1318,11 @@ It is equivalent to
 
   $pssh->push('*', goto => 'END');
 
+=item connect
+
+Just ensures that connecting to the remote machine is possible without
+doing any other action.
+
 =back
 
 When given, C<%opts> can contain the following options:
@@ -1321,8 +1338,8 @@ See L</Error handling>.
 =item or_goto => $tag
 
 Supported for C<command>, C<scp_get>, C<scp_put>, C<rsync_get> and
-C<rsync_put>, when the command, scp or rsync operation fails a goto to
-the given target is performed.
+C<rsync_put>, when the command, C<scp> or C<rsync> operation fails a
+C<goto> to the given target is performed.
 
 For instance:
 
@@ -1372,6 +1389,49 @@ In list context returns a list of pairs C<$label =E<gt> $error> for the failed q
 
 In scalar context returns the number of failed queues.
 
+=back
+
+=head1 FAQ - Frequently Asked Questions
+
+=over 4
+
+=item Running remote commands with sudo
+
+B<Q>: I need to run the remote commands with sudo that asks for a
+password. How can I do it?
+
+B<A>: First read the answer given to a similar question on
+L<Net::OpenSSH> FAQ.
+
+The problem is that Net::OpenSSH::Parallel methods do not support the
+<stdin_data> option, so you will have to use an external file.
+
+  $pssh->push('*', cmd => { stdin_file => $passwd_file },
+                   'sudo', '-Skp', '', '--', @cmd);
+
+One trick you can use if you only have one password is to use the
+C<DATA> file handle:
+
+  $pssh->push('*', cmd => { stdin_fh => \*DATA},
+              'sudo', '-Skp', '', '--', @cmd);
+  ...
+  # and at the end of your script
+  __DATA__
+  this-is-my-remote-password-for-sudo
+
+Or you can also use the C<parsub> action:
+
+  my %sudo_passwords = (host1 => "foo", ...);
+
+  sub sudo {
+    my ($label, $ssh, @cmd) = @_;
+    $ssh->system({stdin_data => "$sudo_passwords{$label}\n"},
+                 'sudo', '-Skp', '', '--', @cmd);
+  }
+
+  $pssh->push('*', parsub => \&sudo, @cmd);
+
+
 =back
 
 =head1 TODO
@@ -1384,14 +1444,14 @@ allow running more than one process per remote server concurrently
 
 =item * delay before reconnect
 
-when connecting fails, do not try to reconnect inmediately but after
+when connecting fails, do not try to reconnect immediately but after
 some predefined period
 
 =item * rationalize debugging
 
 currently it is a mess
 
-=item * add loggin support
+=item * add logging support
 
 log the operations performed in a given file
 
@@ -1417,17 +1477,17 @@ sfandino@yahoo.com), please.
 Feedback and comments are also welcome!
 
 The 'sub' and 'parsub' features should be considered experimental and
-its API or behaviour could be changed in future versions of the
+its API or behavior could be changed in future versions of the
 module.
 
 =head2 Reporting bugs
 
 In order to report a bug, write a minimal program that triggers
-it and place the following line at the beggining:
+it and place the following line at the beginning:
 
   $Net::OpenSSH::Parallel::debug = -1;
 
-Then, send me (via rt or email) the debugging output you get when you
+Then, send me (via RT or email) the debugging output you get when you
 run it. Include also the source code of the script, a description of
 what is going wrong and the details of your OS and the versions of
 Perl, C<Net::OpenSSH> and C<Net::OpenSSH::Parallel> you are using.
@@ -1446,8 +1506,8 @@ requirements and we will get back to you ASAP.
 
 =head2 My wishlist
 
-If you like this module and you're feeling generous, take a look at my
-Amazon Wish List: L<http://amzn.com/w/1WU1P6IR5QZ42>
+If you like this module and you are feeling generous, take a look at
+my Amazon Wish List: L<http://amzn.com/w/1WU1P6IR5QZ42>
 
 Also consider contributing to the OpenSSH project this module builds
 upon: L<http://www.openssh.org/donations.html>.
@@ -1463,17 +1523,24 @@ simpler to use but rather more limited.
 L<GRID::Machine> allows to run perl code distributed in a cluster via
 SSH.
 
-If your application requires orchestating workflows more complex than
+If your application requires orchestrating work-flows more complex than
 those supported by L<Net::OpenSSH::Parallel>, you should probably
-consider some L<POE> based solution (check
+consider some L<POE> or L<AnyEvent> based solution (check
 L<POE::Component::OpenSSH>).
 
 L<App::MrShell> is another module allowing to run the same command in
 several host in parallel.
 
+Some people find easier to use L<Net::OpenSSH> combined with
+L<Parallel::ForkManager>, L<threads> or L<Coro>.
+
+L<Net::SSH::Mechanize> is another framework written on top of
+L<AnyEvent> that allows to run remote commands through SSH in
+parallel.
+
 =head1 COPYRIGHT AND LICENSE
 
-Copyright E<copy> 2009-2011 by Salvador FandiE<ntilde>o
+Copyright E<copy> 2009-2012, 2015 by Salvador FandiE<ntilde>o
 (sfandino@yahoo.com).
 
 This library is free software; you can redistribute it and/or modify
